#include <math.h>
#include <iostream.h>
#include "ReactorConstants.hh"
//
// Simple implementation of Beacom/Vogel inverse beat decac 
// differential cross section.  order=0 means only use (1/M)**0,
// order=1 adds (1/M)**1 corrections.  Notation follows published 
// article
//
// TAB 11/20/03
//
double InverseBeta(double Enu,double CosThetaLab,int order){
  //
  // Cross section constants.  Some for overall scale are just
  // to allow absolute comparison to published articel.
  //
  static ReactorConstants k;
  double pi = k.pi;
  double Mproton = k.Mproton;
  double Delta = k.Delta;
  double Melectron = k.Melectron;
  double Gfermi = k.Gfermi;
  double XCunits = k.XcMeVtoCmsqrd;
  double CosThetaC = (0.9741+0.9756)/2;
  //
  // Radiative correction constant
  //
  double RadCor = 0.024;
  //
  // check for threshold
  //
  double Emin = 
    ((Mproton+Delta+Melectron)*(Mproton+Melectron+Delta)
     -Mproton*Mproton)/2/Mproton;
  if(Enu<Emin)return 0;
  //
  // overall scale
  //
  double Sigma0 = Gfermi*Gfermi*CosThetaC*CosThetaC/pi*(1+RadCor);
  //
  // couplings
  //
  double f = 1.00;
  double f2 = 3.706;
  double g = 1.26;
  //
  // order 0 terms
  //
  double E0 = Enu - Delta;
  if(E0<Melectron)E0=Melectron;
  double p0 = sqrt(E0*E0-Melectron*Melectron);
  double v0 = p0/E0;

  if (order==0){
    double XC = ((f*f+3*g*g)+(f*f-g*g)*v0*CosThetaLab)*p0*E0;
    XC *= Sigma0/2;
    XC *= XCunits;
    return XC;
  }
  //
  //  order 1 terms
  //
  double Ysquared = (Delta*Delta-Melectron*Melectron)/2;
  double E1 = E0*(1-Enu/Mproton*(1-v0*CosThetaLab))-Ysquared/Mproton;
  if(E1<Melectron)E1=Melectron;
  double p1 = sqrt(E1*E1-Melectron*Melectron);
  double v1 = p1/E1;

  double Gamma = 
    2*(f+f2)*g*((2*E0+Delta)*(1-v0*CosThetaLab)-
		Melectron*Melectron/E0)
    +(f*f+g*g)*(Delta*(1+v0*CosThetaLab)+Melectron*Melectron/E0)
    +(f*f+3*g*g)*((E0+Delta)*(1-CosThetaLab/v0)-Delta)
    +(f*f-g*g)*((E0+Delta)*(1-CosThetaLab/v0)-Delta)*v0*CosThetaLab;

  double XC = 
    ((f*f+3*g*g)+(f*f-g*g)*v1*CosThetaLab)*E1*p1
    -Gamma/Mproton*E0*p0;
  XC *= Sigma0/2;
  XC *= XCunits;
  return XC;
  
}
