#ifndef REACTORCONSTANTS_HH
#define REACTORCONSTANTS_HH

class ReactorConstants{
  //
  // Common set of constants for simulation.
  //
public:  
  //  
  // For neutrino event generation
  // XCorder can be changed to 0 to use neglect 1/M effects in the nubar 
  // cross section.
  // Emin,Emax set the range of thrown energy.
  // Rmaxgen is the 
  //
  static const int XCorder = 1;  
  static const double Emin=1.85;
  static const double Emax=8.15;
  static const double Rmaxgen = 200;
  //
  // R0 = Gd loaded radius
  // R1 = unloaded scintillator radius
  // R2 = inactive mineral oil radius
  //
  static const double R0 = 140;
  static const double R1 = 175;
  static const double R2 = 275;
  //
  // basics
  //
  static const double pi = 3.1415928;
  static const double Mproton = 938.27200;
  //
  // Mneutron-Mproton
  //
  static const double Delta = 1.2933318;
  static const double Melectron = 0.510998902;
  //
  // 1/alpha
  //
  static const double alphainv = 137.03599976;
  //
  // Gfermi in MeV**(-2)
  // 
  static const double Gfermi = 1.16639e-11;
  static const double Navogadro = 6.02214199e23;
  //
  //classical electron radius
  //
  static const double Relectron = 2.817940285e-13;  
  //
  // conversion for cross sections from (Mev)**-2 to cm**2.
  //
  static const double XcMeVtoCmsqrd = 0.389379292e-21;
  //
  //  c in cm/ns
  //
  static const double c = 29.9792458;
  //
  // Scintillator constants taken from Chooz.
  // Only H are C are taken into account.
  //
  static const double density = 0.85;
  static const double A0=1;
  static const double A1=12;
  static const double Z0=1;
  static const double Z1=6;
  //
  // Mass fractions
  //
  static const double f0=0.125;
  static const double f1=0.875;
  //
  // Ionization potentials for dE/dx
  //
  static const double I0=19.2e-6;
  static const double I1=78.0e-6;  
  //
  // from the Chooz EPJC article
  // index of refraction
  //
  static const double refracGd = 1.472;
  static const double refracSc = 1.476;
  //
  // neutron mfree pathe in Gd- or un-loaded
  //
  static const double mfpGd =  6;
  static const double mfpSc = 40;
  //
  //  capture times in ns for neutrons
  //  These have been adjusted to give mean captur times of 30.5 us and
  //  180 us in Gd and unloaded Sc, respectively 
  //
  //
  static const double tGd = 24.7e3;
  static const double tSc = 145.8e3;
  //
  // properties of capture in Gd region
  // GdCaptureFraction is the fraction of neutrons captured on Gd
  // The program assumes capture on Gd155 or 157.  Gd155frac is the
  // fraction pf Gd155 capture, etc.
  // Gd155peak is the total energy release from Gd155 capture.
  //
  static const double GdCaptureFraction = 0.841;
  static const double Gd155frac = 0.1848;
  static const double Gd155peak = 8.536;
  static const double Gd157frac = 0.8152;
  static const double Gd157peak = 7.937;
  //
  // This is an ad-hoc guess for the average number of gammas/per Gd.
  // The program in practice generates at least two gammas per capture
  // and an additional Poisson fluctuated GammasPerGd-2.
  //
  static const double GammasPerGd = 3;
  //
  // From capture on H
  // This is the gamma energy from np->D gamma.
  //
  static const double Hpeak = 2.2246;
  //
  // Attenuation lengths in Gd- or un-loaded
  // These are taken from Chooz.
  //
  static const double attenlGd =  400;
  static const double attenlSc = 1000;
  //
  // From Chooz-1 scintillator
  //  
  static const double PhotonsPerMeV = 5300;
  //
  // Positron annihilation times from KAMLAND 
  //    Y. Kino et al.
  //    J. Nucl. Radiochem. Science 1, 63, 200
  // Not yet used.
  //
  static const double ShortPosFrac0 = 0.30;
  static const double ShortPosFrac1 = 0.21;
  static const double LongPosFrac = 0.49;
  static const double ShortPosTime0 = 0.19;
  static const double ShortPosTime1 = 0.48;
  static const double LongPosTime = 3.41;
  //
  // Some PMT properties.
  // 8 inch diameter PMT
  //
  static const double PMTcoverage = 0.2;
  static const double PMTdiameter = 20.16;
  //
  // Quantum efficiency for photostatistics. 
  //
  static const double PMTqe = 0.2;
};
#endif
