#include "ReactorFortran.hh"
#include "ReactorConstants.hh"
#include "ReactorEvent.hh"
#include "ReactorDetector.hh"
#include "ReactorXC.hh"
#include <iostream.h>
#include <math.h>
#include <string.h>

int main(){
  //
  //  Set number of events, paw file name, and ntuple name
  //
  int nevents = 2500;
  char* fileName="th13.ntuple";
  char* ntName="event data";
  //
  // init random number seed
  //
  int luxury = 2;
  int seed = 8796896;
  int k1=0;
  int k2=0;
  rluxgo_(luxury,seed,k1,k2);
  //
  // Paw setup.  The odd calls at the end of normal routines
  // are needed to pass character strings into fortran from C++.
  // The g77 compiler used adds an underscore to fortran routines
  // in making them callable from C++.  A routine with an underscore 
  // in its name gets two underscores at the end.
  //
  hbook_init__();
  //
  int iostat;
  int ntId=1;
  int ntLun=99;
  int ntBuf=1024;
  char* dirName = "TEST    ";
  char* chopt = "N";
  hropen_(ntLun,dirName,fileName,chopt,ntBuf,iostat,
	  strlen(dirName),strlen(fileName),strlen(chopt));
  //
  // define ntuple tags here
  // to add vars, increment ntVars, and add tag to end of list
  // don't change wordsize
  //
  int wordsize = 8;
  int ntVars = 25;
  float tuple[ntVars];
  char ntTags[ntVars*wordsize];
  strcpy(ntTags+ 0*wordsize,"wtc      ");
  strcpy(ntTags+ 1*wordsize,"wtf      ");
  strcpy(ntTags+ 2*wordsize,"enu      ");
  strcpy(ntTags+ 3*wordsize,"KEpos    ");
  strcpy(ntTags+ 4*wordsize,"poscos   ");
  strcpy(ntTags+ 5*wordsize,"KEneu    ");
  strcpy(ntTags+ 6*wordsize,"neucos   ");
  strcpy(ntTags+ 7*wordsize,"vnur     ");
  strcpy(ntTags+ 8*wordsize,"vnucos   ");
  strcpy(ntTags+ 9*wordsize,"vnuphi   ");
  strcpy(ntTags+10*wordsize,"phposvis ");
  strcpy(ntTags+11*wordsize,"phneuvis ");
  strcpy(ntTags+12*wordsize,"vposr    ");
  strcpy(ntTags+13*wordsize,"vposcos  ");
  strcpy(ntTags+14*wordsize,"vposphi  ");
  strcpy(ntTags+15*wordsize,"vneur    ");
  strcpy(ntTags+16*wordsize,"vneucos  ");
  strcpy(ntTags+17*wordsize,"vneuphi  ");
  strcpy(ntTags+18*wordsize,"tpos     ");
  strcpy(ntTags+19*wordsize,"tneu     ");
  strcpy(ntTags+20*wordsize,"npe      ");
  strcpy(ntTags+21*wordsize,"atten    ");
  strcpy(ntTags+22*wordsize,"sfact    ");
  strcpy(ntTags+23*wordsize,"phposrec ");
  strcpy(ntTags+24*wordsize,"phneurec ");

  //
  // set up paw ntuple
  // Last two arguments needed for C++ ==> f strings
  //  
  hbookn_(ntId,ntName,ntVars,dirName,1000,ntTags,
	  strlen(ntName),strlen(dirName),wordsize,ntVars);
  //
  // define some histograms
  // Last argument is need for c++ ==> f strings
  //
  hbook1_(101,"phpos",200,0,0.1,0,strlen("phpos"));
  hbook1_(102,"phneu",200,0,0.1,0,strlen("phneu"));
  //
  hbook1_(201,"tpos",250,0,25,0,strlen("tpos"));
  hbook1_(202,"tneu",250,0,500000,0,strlen("tneu"));
  //
  hbook1_(301,"ngpos",10,0,10,0,strlen("ngpos"));
  hbook1_(302,"ngneu",20,0,20,0,strlen("ngneu"));
  //
  hbook1_(401,"egpos",200,0,10,0,strlen("egpos"));
  hbook1_(402,"egneu",200,0,20,0,strlen("egneu"));
  //
  hbook1_(501,"rgp-rpos",200,0,100,0,strlen("rgp-rpos"));
  hbook1_(502,"rgn-rneu",200,0,100,0,strlen("rgn-rneu"));
  //
  hbook1_(601,"rgp-rnu",200,0,100,0,strlen("rgp-rnu"));
  hbook1_(602,"rgn-rnu",200,0,100,0,strlen("rgn-rnu"));
  //
  hbook1_(701,"tgp-tpos",250,0,5,0,strlen("tgp-tpos"));
  hbook1_(702,"tgn-tneu",250,0,5,0,strlen("tgn-tneu"));
  //
  hbook1_(801,"tgp-tnu",250,0,25,0,strlen("tgp-tnu"));
  hbook1_(802,"tgn-tnu",250,0,1000000,0,strlen("tgn-tnu"));
  //
  // Set up simple detector model
  //
  ReactorDetector Detector;
  //
  // event loop
  //
  for (int events = 0;events<nevents;events++){
    //
    // create neutrino event
    //
    ReactorEvent Event;
    //
    // create fast detector simulation
    //
    Detector.LightsOut(Event);
    //
    // fill ntuple
    //
    tuple[ 0] = Event.GetXCWeight();
    tuple[ 1] = Event.GetFluxWeight();
    tuple[ 2] = Event.GetEnu();
    tuple[ 3] = Event.GetPositronKE();
    tuple[ 4] = Event.GetPositronCosTheta();
    tuple[ 5] = Event.GetNeutronKE();
    tuple[ 6] = Event.GetNeutronCosTheta();
    tuple[ 7] = Event.GetNubarVertexR();
    tuple[ 8] = Event.GetNubarVertexCosTheta();
    tuple[ 9] = Event.GetNubarVertexPhi();
    tuple[10] = Detector.GetPHpositron();
    tuple[11] = Detector.GetPHneutron();
    tuple[12] = Event.GetPositronVertexR();
    tuple[13] = Event.GetPositronVertexCosTheta();
    tuple[14] = Event.GetPositronVertexPhi();
    tuple[15] = Event.GetNeutronVertexR();
    tuple[16] = Event.GetNeutronVertexCosTheta();
    tuple[17] = Event.GetNeutronVertexPhi();
    tuple[18] = Event.GetPositronTime();
    tuple[19] = Event.GetNeutronTime();
    tuple[20] = Detector.GetMeanNpe();
    tuple[21] = Detector.GetMeanAtten();
    tuple[22] = Detector.GetMeanSfactor();
    tuple[23] = Detector.GetPHposReco();
    tuple[24] = Detector.GetPHneuReco();

    hfn_(ntId,tuple);
    //
    // do some looping over all the "PMT"
    //
    int npmt=Detector.GetNpmt();
    for(int n=0;n<npmt;n++){
      int npos;
      int nneu;
      double xyzpmt[3];
      double phpos[100];
      double phneu[100];
      double tpos[100];
      double tneu[100];
      //
      // Get "phototube" information.
      //      
      Detector.GetPMTdata(n,xyzpmt,npos,phpos,tpos,nneu,phneu,tneu);

      for(int nn=0;nn<npos;nn++){
	hfill_(101,*(phpos+nn));
	hfill_(201,*(tpos+nn));
      }
      for(int nn=0;nn<nneu;nn++){
	hfill_(102,*(phneu+nn));
	hfill_(202,*(tneu+nn));
      }
    }
    //
    // Get information about gammas from positron.
    // Note that the positron dedx is defined as a "gamma"
    // originating at the positron range point.
    //
    int nposgamma = Detector.GetNgamma('P'); 
    hfill_(301,nposgamma);
    double egp[nposgamma];
    double tgp[nposgamma];
    double rgp[nposgamma];
    double cgp[nposgamma];
    double fgp[nposgamma];
    Detector.GetGamma('P',egp,tgp,rgp,cgp,fgp);
    for (int n=0;n<nposgamma;n++){
      hfill_(401,egp[n]);
      hfill_(701,tgp[n]-Event.GetPositronTime());
      hfill_(801,tgp[n]-Event.GetNubarTime());
    }
    double xyzp[3*nposgamma];
    Detector.GetGamma('P',xyzp);
    double* pxyz=xyzp;
    for (int n=0;n<nposgamma;n++){
      double* qxyz=Event.GetPositronVertexXYZ();
      double* rxyz=Event.GetNubarVertexXYZ();
      double dr1[3];
      double dr2[3];
      for(int i=0;i<3;i++){
	dr1[i] = *pxyz - *qxyz++;
	dr2[i] = *pxyz++ - *rxyz++;
      }
      hfill_(501,sqrt(dr1[0]*dr1[0]+dr1[1]*dr1[1]+dr1[2]*dr1[2]));
      hfill_(601,sqrt(dr2[0]*dr2[0]+dr2[1]*dr2[1]+dr2[2]*dr2[2]));
    }
    //
    // Get information about gammas from neutron.
    //
    int nneugamma = Detector.GetNgamma('N'); 
    hfill_(302,nneugamma);
    double egn[nneugamma];
    double tgn[nneugamma];
    double rgn[nneugamma];
    double cgn[nneugamma];
    double fgn[nneugamma];
    Detector.GetGamma('N',egn,tgn,rgn,cgn,fgn);
    for (int n=0;n<nneugamma;n++){
      hfill_(402,egn[n]);
      hfill_(702,tgn[n]-Event.GetNeutronTime());
      hfill_(802,tgn[n]-Event.GetNubarTime());
    }
    double xyzn[3*nneugamma];
    Detector.GetGamma('N',xyzn);
    pxyz=xyzn;
    for (int n=0;n<nneugamma;n++){
      double* qxyz=Event.GetNeutronVertexXYZ();
      double* rxyz=Event.GetNubarVertexXYZ();
      double dr1[3];
      double dr2[3];
      for(int i=0;i<3;i++){
	dr1[i] = *pxyz - *qxyz++;
	dr2[i] = *pxyz++ - *rxyz++;
      }
      hfill_(502,sqrt(dr1[0]*dr1[0]+dr1[1]*dr1[1]+dr1[2]*dr1[2]));
      hfill_(602,sqrt(dr2[0]*dr2[0]+dr2[1]*dr2[1]+dr2[2]*dr2[2]));
    }
  }
  //
  // close up paw
  //
  hldir_(" "," ",strlen(" "),strlen(" "));
  char* dirName2 = "//TEST";
  hcdir_(dirName2," ",strlen(dirName2),strlen(" "));
  int outopt = 0;
  int icycle;
  hrout_(outopt,icycle," ",strlen(" "));
  hrend_(dirName,strlen(dirName));
}
